/*
 * Licensed Materials - Use restricted. For use in compiling interfaces to
 * IBM Rational TestManager.
 * (c) Copyright IBM Corporation 2001,2002,2006.  All Rights Reserved.
 */

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <tchar.h>

#include "ExtDefs.h"
#include "proxhdr.h"
#include "psystem.h"
#include "plibdefs.h"
#include "APIRecordingAdapter.h"
// local
#include "APIScriptGeneratorAdapter.h"

#define MAX_ERROR_MSG_SIZE 4096;

static unsigned int uiLibID = 0;
static HANDLE hFile = INVALID_HANDLE_VALUE;
static TCHAR errMsg[MAX_PATH];


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Function Definitions
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int SetOptions(TCHAR *options, 
                       size_t optionsSize,
                       TCHAR* errorMessage,
                       size_t errorMessageSize)
{
#ifdef _DEBUG
    TCHAR buf[1024];
#endif // def _DEBUG
	
	if (optionsSize > RSR_MAX_OPTIONS)
        return RSR_BUFFER_TOO_SHORT;
	
#ifdef _DEBUG
	if (GetEnvironmentVariable(_T("RSR_EXT_STUB_SILENT"), buf, 1024) == 0)
		MessageBox(NULL, options,"API ScriptGenerator Extension SetOptions_: ", MB_OK);
#endif // def _DEBUG

    return RSR_SUCCESS;
}


int GetOptions(TCHAR* options,size_t optionsSize)
{
     _tcscpy(options, _T(""));

    if (GetEnvironmentVariable(_T("RSR_ScriptGenerator_EXTENSION_OPTIONS"), options, 1024) != 0)
    {    
        if(_tcslen(options)>0)
			return RSR_SUCCESS;
    }

    _tcscat(options, "option1;option2");
		return RSR_SUCCESS;
    
    if (_tcslen(options)+1 > optionsSize)
        return RSR_BUFFER_TOO_SHORT;
    else
        return RSR_SUCCESS;
}

BOOL IsAPISgenAdapter()
{
	return TRUE;
}

int InitializeScriptgen(TCHAR* scriptPathName)
{
	DWORD dwWritten;
#ifdef _DEBUG
	MessageBox(NULL, scriptPathName, "HelloWorld InitializeScriptgen", MB_OK);
#endif // def _DEBUG
	hFile = CreateFile(scriptPathName,
			GENERIC_READ | GENERIC_WRITE,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			CREATE_ALWAYS,
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
			NULL);


	if (hFile != INVALID_HANDLE_VALUE)
	{
#ifdef _DEBUG
		MessageBox(NULL, scriptPathName, "HelloWorld Script File Created OK", MB_OK);
#endif // def _DEBUG
		strcpy(errMsg, "script file created");

        WriteFile(hFile, (void*)"\r\n", strlen("\r\n"), &dwWritten, NULL);

        WriteFile(hFile, (void*)"external_C int func HelloWorld1(s)\r\n", strlen("external_C int func HelloWorld1(s)\r\n"), &dwWritten, NULL);
        WriteFile(hFile, (void*)"string s;\r\n", strlen("string s;\r\n"), &dwWritten, NULL);
        WriteFile(hFile, (void*)"{}\r\n\r\n", strlen("{}\r\n\r\n"), &dwWritten, NULL);

        WriteFile(hFile, (void*)"external_C int func HelloWorld2(s)\r\n", strlen("external_C int func HelloWorld1(s)\r\n"), &dwWritten, NULL);
        WriteFile(hFile, (void*)"string s;\r\n", strlen("string s;\r\n"), &dwWritten, NULL);
        WriteFile(hFile, (void*)"{}\r\n\r\n", strlen("{}\r\n\r\n"), &dwWritten, NULL);

        WriteFile(hFile, (void*)"{\r\n", strlen("{\r\n"), &dwWritten, NULL);

		return RSR_OK;
	}
	else
	{
		strcpy(errMsg, "File open failed in HelloWorld Script Gen");
	}

	return RSR_ERROR;
		
}

int GetStatus(TCHAR* msg, int *msgSize)
{
	strcpy(msg, errMsg);

	return RSR_IGNORE;

}

int PassComplete(int passNumber)
{
	DWORD dwWritten;
    WriteFile(hFile, (void*)"}\r\n", strlen("}\r\n"), &dwWritten, NULL);

	CloseHandle(hFile);
	hFile = INVALID_HANDLE_VALUE;

	strcpy(errMsg, "script gen complete");

	return RSR_IGNORE;
}


int ProcessAPIPacket(void* packet,
                        int  connNum)
{
 
	major_wch_header *majorhdr;
	int IDNum;
#ifdef _DEBUG
	char buf[MAX_PATH];
#endif // def _DEBUG

	majorhdr = packet;
	

	// this example doesn't pass any packet type besides API_SINGLE
	// on to the script generator
#ifdef _DEBUG
	MessageBox(NULL, "packet", "HelloWorld ScriptGenerator", MB_OK);
#endif // def _DEBUG
	if (majorhdr->major_type == API_SINGLE)
	{
		minor_wch_header *minorhdr = (minor_wch_header *)((char*)packet + sizeof(major_wch_header));
		IDNum = minorhdr->library_id;

#ifdef _DEBUG
		MessageBox(NULL, "API_SINGLE OK", "HelloWorld ScriptGenerator", MB_OK);
#endif // def _DEBUG
		// I'm interested in the following packet types
		switch (IDNum)
		{	
			// add ScriptGenerators for other libraries here
			case PROXY_LIB_WINDOWS_SYSTEM:
					// check all the customer adapter records
					// to see if it is ours
					if (minorhdr->API_id == P_CUSTOMADAPTER_A)
					{
						// custom adapter record, let's
						// see if it is ours
						S_CUSTOMADAPTER *custadapter = (S_CUSTOMADAPTER *)((char*)packet + sizeof(major_wch_header)
							+ sizeof(minor_wch_header)  + sizeof(stamp2_wch_header));
						void *p = (void *)((char*)packet + sizeof(major_wch_header)
							+ sizeof(minor_wch_header) + sizeof(stamp2_wch_header) + custadapter->iOffszLibraryName);
						char *szLibName = (char *)p;

#ifdef _DEBUG
						MessageBox(NULL, "custadapterpacket found", "HelloWorld ScriptGenerator adapter", MB_OK);
#endif // def _DEBUG
						if (stricmp(szLibName, "APIRecordingAdapter.dll") == 0)
						{
							uiLibID = custadapter->uiLibID;
#ifdef _DEBUG
							MessageBox(NULL, szLibName, "Identified System library record",  MB_OK);
							wsprintf(buf, "lib ID=%d", uiLibID);
							MessageBox(NULL, buf, "HelloWorld ScriptGenerator adapter", MB_OK);
#endif // def _DEBUG
						}
					}
					break;
			default:
					// check for any records that are to be thrown away
#ifdef _DEBUG
					wsprintf(buf,"default processing libID=%d looking for uiLibID=%d",IDNum, uiLibID);
					MessageBox(NULL,buf,"HelloWorld non sys packet", MB_OK);
#endif // def _DEBUG

					if (IDNum == (int)uiLibID)
					{
						S_HELLOWORLD *sHello;
						char buf[MAX_PATH];
						char *szStr;
						void *p = (void *)((char*)packet + sizeof(major_wch_header)
							+ sizeof(minor_wch_header) + sizeof(stamp2_wch_header));
						DWORD dwWritten;
						char *functionName = "";
						sHello = (S_HELLOWORLD *)p;
						szStr = (char *)((char *)p + sHello->iOffszStr);

#ifdef _DEBUG
						wsprintf(buf,"processing with good data packet LibID=%d", uiLibID);
						MessageBox(NULL,buf,"HelloWorld ScriptGenerator", MB_OK);
#endif // def _DEBUG

						switch (minorhdr->API_id) {
						case P_HELLOWORLD1:
							functionName = "HelloWorld1";
							break;
						case P_HELLOWORLD2:
							functionName = "HelloWorld2";
							break;
						default:
#ifdef _DEBUG
							wsprintf(buf,"Invalid API id: %d", minorhdr->API_id);
							MessageBox(NULL, "Invalid API id","HelloWorld ScriptGenerator", MB_OK);
#endif // def _DEBUG

							wsprintf(buf,"/* Invalid API id: %d */", minorhdr->API_id);
	    					WriteFile(hFile, (void*)buf, strlen(buf), &dwWritten, NULL);
    						WriteFile(hFile, (void*)"\r\n", strlen("\r\n"), &dwWritten, NULL);

							return RSR_ERROR;
							break;
						}

#ifdef _DEBUG
						wsprintf(buf,"API id = %d, name = %s", minorhdr->API_id, functionName);
						MessageBox(NULL,buf,"HelloWorld ScriptGenerator", MB_OK);
						wsprintf(buf,"return value = %d, Msg: %s", sHello->bReturn, szStr);
						MessageBox(NULL,buf,"HelloWorld ScriptGenerator", MB_OK);
#endif // def _DEBUG

						wsprintf(buf,"/* return value = %d, Msg: %s */", sHello->bReturn, szStr);
						WriteFile(hFile, (void*)buf, strlen(buf), &dwWritten, NULL);
						WriteFile(hFile, (void*)"\r\n", strlen("\r\n"), &dwWritten, NULL);

						wsprintf(buf, "testcase (%s(\"%s\") == %d),\r\n", functionName, szStr, sHello->bReturn);
						WriteFile(hFile, (void*)buf, strlen(buf), &dwWritten, NULL);

						wsprintf(buf, "    \"Called %s(\\\"%s\\\")\";\r\n", functionName, szStr);
						WriteFile(hFile, (void*)buf, strlen(buf), &dwWritten, NULL);

						WriteFile(hFile, (void*)"\r\n", strlen("\r\n"), &dwWritten, NULL);
					}
					break;
					
		}

	}


	return RSR_OK;
}


